@@ -2,6 +2,7 @@
 
 #include <stdbool.h>
 #include <stdint.h>
+#include <netinet/in.h>
 
 #include <osmocom/core/timer.h>
 #include <osmocom/core/linuxlist.h>
@@ -14,6 +15,7 @@
 #include <osmocom/gsm/gsm48_rest_octets.h>
 #include <osmocom/gsm/protocol/gsm_04_08.h>
 #include <osmocom/gsm/meas_rep.h>
+#include <osmocom/netif/osmux.h>
 
 #include <osmo-bts/power_control.h>
 
@@ -162,6 +164,18 @@ struct gsm_lchan {
 		uint8_t rtp_payload;
 		uint8_t rtp_payload2;
 		uint8_t speech_mode;
+		struct {
+			bool use;
+			uint8_t local_cid;
+			uint8_t remote_cid;
+			/* Rx Osmux -> RTP, one allocated & owned per lchan */
+			struct osmux_out_handle *out;
+			 /* Tx RTP -> Osmux, shared by all lchans sharing a
+			  * remote endp (addr+port), see "struct osmux_handle" */
+			struct osmux_in_handle *in;
+			/* Used to build rtp messages we send to osmux */
+			struct osmo_rtp_handle *rtpst;
+		} osmux;
 		struct osmo_rtp_socket *rtp_socket;
 	} abis_ip;
 
@@ -190,6 +204,7 @@ struct gsm_lchan {
 	uint8_t sapis_ul[23];
 	struct lapdm_channel lapdm_ch;
 	struct llist_head dl_tch_queue;
+	unsigned int dl_tch_queue_len;
 	struct {
 		/* bitmask of all SI that are present/valid in si_buf */
 		uint32_t valid;
@@ -354,6 +369,12 @@ int lchan2ecu_codec(const struct gsm_lchan *lchan);
 
 void lchan_set_state(struct gsm_lchan *lchan, enum gsm_lchan_state state);
 
+int lchan_rtp_socket_create(struct gsm_lchan *lchan, const char *bind_ip);
+int lchan_rtp_socket_connect(struct gsm_lchan *lchan, const struct in_addr *ia, uint16_t connect_port);
+void lchan_rtp_socket_free(struct gsm_lchan *lchan);
+
+void lchan_dl_tch_queue_enqueue(struct gsm_lchan *lchan, struct msgb *msg, unsigned int limit);
+
 static inline bool lchan_is_dcch(const struct gsm_lchan *lchan)
 {
 	switch (lchan->type) {
