@@ -20,7 +20,7 @@ enum {
 	DLOOP,
 	DABIS,
 	DRTP,
-	DSUM,
+	DOSMUX,
 };
 
 extern const struct log_info bts_log_info;
