@@ -54,6 +54,7 @@
 #include <osmo-bts/bts_shutdown_fsm.h>
 #include <osmo-bts/nm_common_fsm.h>
 #include <osmo-bts/power_control.h>
+#include <osmo-bts/osmux.h>
 
 #define MIN_QUAL_RACH	 50 /* minimum link quality (in centiBels) for Access Bursts */
 #define MIN_QUAL_NORM	 -5 /* minimum link quality (in centiBels) for Normal Bursts */
@@ -219,6 +220,12 @@ static int gsm_bts_talloc_destructor(struct gsm_bts *bts)
 		osmo_fsm_inst_free(bts->mo.fi);
 		bts->mo.fi = NULL;
 	}
+	if (bts->shutdown_fi) {
+		osmo_fsm_inst_free(bts->shutdown_fi);
+		bts->shutdown_fi = NULL;
+	}
+
+	bts_osmux_release(bts);
 	return 0;
 }
 
@@ -368,18 +375,28 @@ int bts_init(struct gsm_bts *bts)
 	oml_mo_state_init(&bts->gprs.nse.mo, NM_OPSTATE_DISABLED, NM_AVSTATE_DEPENDENCY);
 	oml_mo_state_init(&bts->gprs.cell.mo, NM_OPSTATE_DISABLED, NM_AVSTATE_DEPENDENCY);
 	oml_mo_state_init(&bts->gprs.nsvc[0].mo, NM_OPSTATE_DISABLED, NM_AVSTATE_DEPENDENCY);
-	oml_mo_state_init(&bts->gprs.nsvc[1].mo, NM_OPSTATE_DISABLED, NM_AVSTATE_OFF_LINE);
+	oml_mo_state_init(&bts->gprs.nsvc[1].mo, NM_OPSTATE_DISABLED, NM_AVSTATE_DEPENDENCY);
 
 	/* allocate a talloc pool for ORTP to ensure it doesn't have to go back
 	 * to the libc malloc all the time */
 	tall_rtp_ctx = talloc_pool(tall_bts_ctx, 262144);
 	osmo_rtp_init(tall_rtp_ctx);
 
-	/* features implemented in 'common', available for all models */
+	/* Osmux */
+	rc = bts_osmux_init(bts);
+	if (rc < 0) {
+		llist_del(&bts->list);
+		return rc;
+	}
+
+	/* features implemented in 'common', available for all models,
+	 * order alphabetically */
+	osmo_bts_set_feature(bts->features, BTS_FEAT_ABIS_OSMO_PCU);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_CCN);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_DYN_TS_SDCCH8);
 	osmo_bts_set_feature(bts->features, BTS_FEAT_ETWS_PN);
 	osmo_bts_set_feature(bts->features, BTS_FEAT_IPV6_NSVC);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_ABIS_OSMO_PCU);
-	osmo_bts_set_feature(bts->features, BTS_FEAT_DYN_TS_SDCCH8);
+	osmo_bts_set_feature(bts->features, BTS_FEAT_PAGING_COORDINATION);
 
 	rc = bts_model_init(bts);
 	if (rc < 0) {
@@ -425,7 +442,7 @@ int bts_link_estab(struct gsm_bts *bts)
 {
 	int i, j;
 
-	LOGP(DSUM, LOGL_INFO, "Main link established, sending NM Status.\n");
+	LOGP(DOML, LOGL_INFO, "Main link established, sending NM Status.\n");
 
 	/* BTS SITE MGR becomes Offline (tx SW ACT Report), BTS is DEPENDENCY */
 	osmo_fsm_inst_dispatch(bts->site_mgr.mo.fi, NM_EV_SW_ACT, NULL);
@@ -648,7 +665,7 @@ int bts_agch_enqueue(struct gsm_bts *bts, struct msgb *msg)
 	struct gsm48_imm_ass_rej *imm_ass_cmd = msgb_l3(msg);
 
 	if (bts->agch_queue.length > hard_limit) {
-		LOGP(DSUM, LOGL_ERROR,
+		LOGP(DRR, LOGL_ERROR,
 		     "AGCH: too many messages in queue, "
 		     "refusing message type %s, length = %d/%d\n",
 		     gsm48_rr_msg_name(((struct gsm48_imm_ass *)msgb_l3(msg))->msg_type),
@@ -879,22 +896,23 @@ int bts_supports_cm(const struct gsm_bts *bts,
 /* return the gsm_lchan for the CBCH (if it exists at all) */
 struct gsm_lchan *gsm_bts_get_cbch(struct gsm_bts *bts)
 {
-	struct gsm_lchan *lchan = NULL;
 	struct gsm_bts_trx *trx = bts->c0;
 
+	/* According to 3GPP TS 45.002, table 3, CBCH can be allocated
+	 * either on C0/TS0 (CCCH+SDCCH4) or on C0..n/TS0..3 (SDCCH/8). */
 	if (trx->ts[0].pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH)
-		lchan = &trx->ts[0].lchan[2];
-	else {
-		int i;
-		for (i = 0; i < 8; i++) {
-			if (trx->ts[i].pchan == GSM_PCHAN_SDCCH8_SACCH8C_CBCH) {
-				lchan = &trx->ts[i].lchan[2];
-				break;
-			}
+		return &trx->ts[0].lchan[2]; /* C0/TS0 */
+
+	llist_for_each_entry(trx, &bts->trx_list, list) { /* C0..n */
+		unsigned int tn;
+		for (tn = 0; tn <= 3; tn++) { /* TS0..3 */
+			struct gsm_bts_trx_ts *ts = &trx->ts[tn];
+			if (ts->pchan == GSM_PCHAN_SDCCH8_SACCH8C_CBCH)
+				return &ts->lchan[2];
 		}
 	}
 
-	return lchan;
+	return NULL;
 }
 
 /* BCCH carrier power reduction (see 3GPP TS 45.008, section 7.1) */
