@@ -152,27 +152,13 @@ static uint32_t fn_ms_adj(uint32_t fn, const struct gsm_lchan *lchan)
 	return GSM_RTP_DURATION;
 }
 
-/*! limit number of queue entries to %u; drops any surplus messages */
-static void queue_limit_to(const char *prefix, struct llist_head *queue, unsigned int limit)
-{
-	unsigned int count = llist_count(queue);
-
-	if (count > limit)
-		LOGP(DL1P, LOGL_NOTICE, "%s: freeing %d queued frames\n", prefix, count-limit);
-	while (count > limit) {
-		struct msgb *tmp = msgb_dequeue(queue);
-		msgb_free(tmp);
-		count--;
-	}
-}
-
 /* allocate a msgb containing a osmo_phsap_prim + optional l2 data
  * in order to wrap femtobts header around l2 data, there must be enough space
  * in front and behind data pointer */
 struct msgb *l1sap_msgb_alloc(unsigned int l2_len)
 {
-	int headroom = 128;
-	int size = headroom + sizeof(struct osmo_phsap_prim) + l2_len;
+	const int headroom = L1SAP_MSGB_HEADROOM;
+	const int size = headroom + sizeof(struct osmo_phsap_prim) + l2_len;
 	struct msgb *msg = msgb_alloc_headroom(size, headroom, "l1sap_prim");
 
 	if (!msg)
@@ -363,7 +349,7 @@ static int gsmtap_ph_data(const struct osmo_phsap_prim *l1sap,
 	uint8_t chan_nr, link_id;
 
 	*data = msgb_l2(msg);
-	*len = msgb_l2len(msg);
+	*len = msgb_l2(msg) ? msgb_l2len(msg) : 0;
 	chan_nr = l1sap->u.data.chan_nr;
 	link_id = l1sap->u.data.link_id;
 
@@ -405,7 +391,7 @@ static int gsmtap_pdch(const struct osmo_phsap_prim *l1sap,
 	struct msgb *msg = l1sap->oph.msg;
 
 	*data = msgb_l2(msg);
-	*len = msgb_l2len(msg);
+	*len = msgb_l2(msg) ? msgb_l2len(msg) : 0;
 
 	if (L1SAP_IS_PTCCH(fn)) {
 		*chan_type = GSMTAP_CHANNEL_PTCCH;
@@ -697,6 +683,8 @@ static inline void set_ms_to_data(struct gsm_lchan *lchan, int16_t data, bool se
 	}
 }
 
+bool trx_sched_is_sacch_fn(const struct gsm_bts_trx_ts *ts, uint32_t fn, bool uplink);
+
 /* measurement information received from bts model */
 static void process_l1sap_meas_data(struct gsm_lchan *lchan,
 				    const struct osmo_phsap_prim *l1sap,
@@ -719,9 +707,11 @@ static void process_l1sap_meas_data(struct gsm_lchan *lchan,
 			.ta_offs_256bits = info_meas_ind->ta_offs_256bits,
 			.inv_rssi = info_meas_ind->inv_rssi,
 			.ber10k = info_meas_ind->ber10k,
-			.c_i = info_meas_ind->c_i_cb,
-			.is_sub = info_meas_ind->is_sub,
+			.ci_cb = info_meas_ind->c_i_cb,
 		};
+		/* additionally treat SACCH frames (match by TDMA FN) as SUB frames */
+		if (info_meas_ind->is_sub || trx_sched_is_sacch_fn(lchan->ts, fn, true))
+			ulm.is_sub = 1;
 		break;
 	case PRIM_TCH:
 		ph_tch_ind = &l1sap->u.tch;
@@ -733,9 +723,10 @@ static void process_l1sap_meas_data(struct gsm_lchan *lchan,
 			.ta_offs_256bits = ph_tch_ind->ta_offs_256bits,
 			.inv_rssi = abs(ph_tch_ind->rssi),
 			.ber10k = ph_tch_ind->ber10k,
-			.c_i = ph_tch_ind->lqual_cb,
+			.ci_cb = ph_tch_ind->lqual_cb,
 			.is_sub = ph_tch_ind->is_sub,
 		};
+		/* PRIM_TCH always carries DCCH, not SACCH */
 		break;
 	case PRIM_PH_DATA:
 		ph_data_ind = &l1sap->u.data;
@@ -747,9 +738,11 @@ static void process_l1sap_meas_data(struct gsm_lchan *lchan,
 			.ta_offs_256bits = ph_data_ind->ta_offs_256bits,
 			.inv_rssi = abs(ph_data_ind->rssi),
 			.ber10k = ph_data_ind->ber10k,
-			.c_i = ph_data_ind->lqual_cb,
-			.is_sub = ph_data_ind->is_sub,
+			.ci_cb = ph_data_ind->lqual_cb,
 		};
+		/* additionally treat SACCH frames (match by RSL link ID) as SUB frames */
+		if (ph_data_ind->is_sub || L1SAP_IS_LINK_SACCH(ph_data_ind->link_id))
+			ulm.is_sub = 1;
 		break;
 	default:
 		OSMO_ASSERT(false);
@@ -758,7 +751,7 @@ static void process_l1sap_meas_data(struct gsm_lchan *lchan,
 	DEBUGPFN(DL1P, fn,
 		 "%s %s meas ind, ta_offs_256bits=%d, ber10k=%d, inv_rssi=%u, C/I=%d cB\n",
 		 gsm_lchan_name(lchan), ind_name, ulm.ta_offs_256bits,
-		 ulm.ber10k, ulm.inv_rssi, ulm.c_i);
+		 ulm.ber10k, ulm.inv_rssi, ulm.ci_cb);
 
 	/* we assume that symbol period is 1 bit: */
 	set_ms_to_data(lchan, ulm.ta_offs_256bits / 256, true);
@@ -908,7 +901,7 @@ static int lchan_pdtch_ph_rts_ind_loop(struct gsm_lchan *lchan,
 	uint8_t *p;
 
 	/* de-queue response message (loopback) */
-	loop_msg = msgb_dequeue(&lchan->dl_tch_queue);
+	loop_msg = msgb_dequeue_count(&lchan->dl_tch_queue, &lchan->dl_tch_queue_len);
 	if (!loop_msg) {
 		LOGPGT(DL1P, LOGL_NOTICE, tm, "%s: no looped PDTCH message, sending empty\n",
 		     gsm_lchan_name(lchan));
@@ -1243,7 +1236,7 @@ static bool rtppayload_is_octet_aligned(const uint8_t *rtp_pl, uint8_t payload_l
 	#define AMR_PADDING1(rtp_pl) (rtp_pl[0] & 0x0f)
 	#define AMR_PADDING2(rtp_pl) (rtp_pl[1] & 0x03)
 
-	if(payload_len < 2 || AMR_PADDING1(rtp_pl) || AMR_PADDING2(rtp_pl))
+	if (payload_len < 2 || AMR_PADDING1(rtp_pl) || AMR_PADDING2(rtp_pl))
 		return false;
 
 	return true;
@@ -1253,7 +1246,7 @@ static bool rtppayload_is_valid(struct gsm_lchan *lchan, struct msgb *resp_msg)
 {
 	/* Avoid sending bw-efficient AMR to lower layers, most bts models
 	 * don't support it. */
-	if(lchan->tch_mode == GSM48_CMODE_SPEECH_AMR &&
+	if (lchan->tch_mode == GSM48_CMODE_SPEECH_AMR &&
 		!rtppayload_is_octet_aligned(resp_msg->data, resp_msg->len)) {
 		LOGPLCHAN(lchan, DL1P, LOGL_NOTICE,
 			  "RTP->L1: Dropping unexpected AMR encoding (bw-efficient?) %s\n",
@@ -1298,11 +1291,11 @@ static int l1sap_tch_rts_ind(struct gsm_bts_trx *trx,
 		lchan->abis_ip.rtp_socket->rx_user_ts += GSM_RTP_DURATION;
 	}
 	/* get a msgb from the dl_tx_queue */
-	resp_msg = msgb_dequeue(&lchan->dl_tch_queue);
+	resp_msg = msgb_dequeue_count(&lchan->dl_tch_queue, &lchan->dl_tch_queue_len);
 	if (!resp_msg) {
 		DEBUGPGT(DL1P, &g_time, "%s DL TCH Tx queue underrun\n", gsm_lchan_name(lchan));
 		resp_l1sap = &empty_l1sap;
-	} else if(!rtppayload_is_valid(lchan, resp_msg)) {
+	} else if (!rtppayload_is_valid(lchan, resp_msg)) {
 		msgb_free(resp_msg);
 		resp_msg = NULL;
 		resp_l1sap = &empty_l1sap;
@@ -1493,8 +1486,7 @@ static int l1sap_ph_data_ind(struct gsm_bts_trx *trx,
 			/* we are in loopback mode (for BER testing)
 			 * mode and need to enqeue the frame to be
 			 * returned in downlink */
-			queue_limit_to(gsm_lchan_name(lchan), &lchan->dl_tch_queue, 1);
-			msgb_enqueue(&lchan->dl_tch_queue, msg);
+			lchan_dl_tch_queue_enqueue(lchan, msg, 1);
 
 			/* Return 1 to signal that we're still using msg
 			 * and it should not be freed */
@@ -1610,15 +1602,17 @@ static int l1sap_tch_ind(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap,
 	 * good enough. */
 	if (msg->len && tch_ind->lqual_cb >= bts->min_qual_norm) {
 		/* hand msg to RTP code for transmission */
-		if (lchan->abis_ip.rtp_socket)
+		if (lchan->abis_ip.osmux.use) {
+			lchan_osmux_send_frame(lchan, msg->data, msg->len,
+					       fn_ms_adj(fn, lchan), lchan->rtp_tx_marker);
+		} else if (lchan->abis_ip.rtp_socket) {
 			osmo_rtp_send_frame_ext(lchan->abis_ip.rtp_socket,
 				msg->data, msg->len, fn_ms_adj(fn, lchan), lchan->rtp_tx_marker);
+		}
 		/* if loopback is enabled, also queue received RTP data */
 		if (lchan->loopback) {
-			/* make sure the queue doesn't get too long */
-			queue_limit_to(gsm_lchan_name(lchan), &lchan->dl_tch_queue, 1);
-			/* add new frame to queue */
-			msgb_enqueue(&lchan->dl_tch_queue, msg);
+			/* add new frame to queue, make sure the queue doesn't get too long */
+			lchan_dl_tch_queue_enqueue(lchan, msg, 1);
 			/* Return 1 to signal that we're still using msg and it should not be freed */
 			return 1;
 		}
@@ -1627,7 +1621,9 @@ static int l1sap_tch_ind(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap,
 	} else {
 		DEBUGPGT(DRTP, &g_time, "Skipping RTP frame with lost payload (chan_nr=0x%02x)\n",
 			 chan_nr);
-		if (lchan->abis_ip.rtp_socket)
+		if (lchan->abis_ip.osmux.use)
+			lchan_osmux_skipped_frame(lchan, fn_ms_adj(fn, lchan));
+		else if (lchan->abis_ip.rtp_socket)
 			osmo_rtp_skipped_frame(lchan->abis_ip.rtp_socket, fn_ms_adj(fn, lchan));
 		lchan->rtp_tx_marker = true;
 	}
@@ -1886,7 +1882,6 @@ void l1sap_rtp_rx_cb(struct osmo_rtp_socket *rs, const uint8_t *rtp_pl,
 {
 	struct gsm_lchan *lchan = rs->priv;
 	struct msgb *msg;
-	struct osmo_phsap_prim *l1sap;
 
 	/* if we're in loopback mode, we don't accept frames from the
 	 * RTP socket anymore */
@@ -1897,7 +1892,7 @@ void l1sap_rtp_rx_cb(struct osmo_rtp_socket *rs, const uint8_t *rtp_pl,
 	if (!msg)
 		return;
 	memcpy(msgb_put(msg, rtp_pl_len), rtp_pl, rtp_pl_len);
-	msgb_pull(msg, sizeof(*l1sap));
+	msgb_pull(msg, sizeof(struct osmo_phsap_prim));
 
 	/* Store RTP header Marker bit in control buffer */
 	rtpmsg_marker_bit(msg) = marker;
@@ -1907,9 +1902,7 @@ void l1sap_rtp_rx_cb(struct osmo_rtp_socket *rs, const uint8_t *rtp_pl,
 	rtpmsg_ts(msg) = timestamp;
 
 	/* make sure the queue doesn't get too long */
-	queue_limit_to(gsm_lchan_name(lchan), &lchan->dl_tch_queue, 1);
-
-	msgb_enqueue(&lchan->dl_tch_queue, msg);
+	lchan_dl_tch_queue_enqueue(lchan, msg, 1);
 }
 
 static int l1sap_chan_act_dact_modify(struct gsm_bts_trx *trx, uint8_t chan_nr,
@@ -1927,7 +1920,7 @@ static int l1sap_chan_act_dact_modify(struct gsm_bts_trx *trx, uint8_t chan_nr,
 	return l1sap_down(trx, &l1sap);
 }
 
-int l1sap_chan_act(struct gsm_bts_trx *trx, uint8_t chan_nr, struct tlv_parsed *tp)
+int l1sap_chan_act(struct gsm_bts_trx *trx, uint8_t chan_nr)
 {
 	struct gsm_lchan *lchan = get_lchan_by_chan_nr(trx, chan_nr);
 	int rc;
@@ -1947,7 +1940,7 @@ int l1sap_chan_act(struct gsm_bts_trx *trx, uint8_t chan_nr, struct tlv_parsed *
 		return -RSL_ERR_EQUIPMENT_FAIL;
 
 	/* Init DTX DL FSM if necessary */
-	if (trx->bts->dtxd && lchan->type != GSM_LCHAN_SDCCH) {
+	if (trx->bts->dtxd && lchan_is_tch(lchan)) {
 		lchan->tch.dtx.dl_amr_fsm = osmo_fsm_inst_alloc(&dtx_dl_amr_fsm,
 								tall_bts_ctx,
 								lchan,
