@@ -233,7 +233,7 @@ static void signal_handler(int signum)
 			oml_tx_failure_event_rep(&g_bts->mo,
 						 NM_SEVER_CRITICAL, OSMO_EVT_CRIT_PROC_STOP,
 						 "BTS: SIGINT received -> shutdown");
-			bts_shutdown(g_bts, "SIGINT");
+			bts_shutdown_ext(g_bts, "SIGINT", true, false);
 		}
 		quit++;
 		break;
@@ -257,25 +257,6 @@ static void signal_handler(int signum)
 	}
 }
 
-static int write_pid_file(char *procname)
-{
-	FILE *outf;
-	char tmp[PATH_MAX+1];
-
-	snprintf(tmp, sizeof(tmp)-1, "/var/run/%s.pid", procname);
-	tmp[PATH_MAX-1] = '\0';
-
-	outf = fopen(tmp, "w");
-	if (!outf)
-		return -1;
-
-	fprintf(outf, "%d\n", getpid());
-
-	fclose(outf);
-
-	return 0;
-}
-
 int bts_main(int argc, char **argv)
 {
 	struct gsm_bts_trx *trx;
@@ -303,6 +284,8 @@ int bts_main(int argc, char **argv)
 	bts_vty_init(tall_bts_ctx);
 	e1inp_vty_init();
 
+	logging_vty_add_deprecated_subsys(tall_bts_ctx, "sum");
+
 	handle_options(argc, argv);
 
 	fprintf(stderr, "((*))\n  |\n / \\ OsmoBTS\n");
@@ -356,8 +339,6 @@ int bts_main(int argc, char **argv)
 		}
 	}
 
-	write_pid_file("osmo-bts");
-
 	/* Accept a GSMTAP host from VTY config, but a commandline option overrides that. */
 	if (gsmtap_ip != NULL) {
 		if (g_bts->gsmtap.remote_host != NULL) {
@@ -384,10 +365,9 @@ int bts_main(int argc, char **argv)
 		gsmtap_source_add_sink(g_bts->gsmtap.inst);
 	}
 
-	bts_controlif_setup(g_bts, ctrl_vty_get_bind_addr(), OSMO_CTRL_PORT_BTS);
+	bts_controlif_setup(g_bts, OSMO_CTRL_PORT_BTS);
 
-	rc = telnet_init_dynif(tall_bts_ctx, NULL, vty_get_bind_addr(),
-			       g_vty_port_num);
+	rc = telnet_init_default(tall_bts_ctx, NULL, g_vty_port_num);
 	if (rc < 0) {
 		fprintf(stderr, "Error initializing telnet\n");
 		exit(1);
@@ -405,6 +385,11 @@ int bts_main(int argc, char **argv)
 	signal(SIGUSR2, &signal_handler);
 	osmo_init_ignore_signals();
 
+	if (bts_osmux_open(g_bts) < 0) {
+		fprintf(stderr, "Osmux setup failed\n");
+		exit(1);
+	}
+
 	if (vty_test_mode) {
 		/* Just select-loop without connecting to the BSC, don't exit. This allows running tests on the VTY
 		 * telnet port. */
