@@ -203,7 +203,7 @@ static void info_ind_fill_trx(struct gsm_pcu_if_info_trx *trx_info,
 		return;
 	}
 
-	for (tn = 0; tn < 8; tn++) {
+	for (tn = 0; tn < ARRAY_SIZE(trx->ts); tn++) {
 		const struct gsm_bts_trx_ts *ts = &trx->ts[tn];
 
 		if (ts->mo.nm_state.operational != NM_OPSTATE_ENABLED)
@@ -899,7 +899,7 @@ static int pcu_rx_act_req(struct gsm_bts *bts,
 			     "size is %zu, discarding\n", prim_len, #prim_msg, _len); \
 			return -EINVAL; \
 		} \
-	} while(0);
+	} while (0)
 static int pcu_rx(struct gsm_network *net, uint8_t msg_type,
 	struct gsm_pcu_if *pcu_prim, size_t prim_len)
 {
