@@ -659,21 +659,10 @@ static void trx_sched_clean_ts(struct gsm_bts_trx_ts *ts)
 	struct l1sched_ts *l1ts = ts->priv;
 	unsigned int i;
 
-	msgb_queue_flush(&l1ts->dl_prims);
+	msgb_queue_free(&l1ts->dl_prims);
 	rate_ctr_group_free(l1ts->ctrs);
 	l1ts->ctrs = NULL;
-	for (i = 0; i < _TRX_CHAN_MAX; i++) {
-		struct l1sched_chan_state *chan_state;
-		chan_state = &l1ts->chan_state[i];
-		if (chan_state->dl_bursts) {
-			talloc_free(chan_state->dl_bursts);
-			chan_state->dl_bursts = NULL;
-		}
-		if (chan_state->ul_bursts) {
-			talloc_free(chan_state->ul_bursts);
-			chan_state->ul_bursts = NULL;
-		}
-	}
+
 	/* clear lchan channel states */
 	for (i = 0; i < ARRAY_SIZE(ts->lchan); i++)
 		lchan_set_state(&ts->lchan[i], LCHAN_S_NONE);
@@ -799,9 +788,6 @@ int _sched_compose_ph_data_ind(struct l1sched_ts *l1ts, uint32_t fn,
 	if (l2_len)
 		memcpy(msg->l2h, l2, l2_len);
 
-	if (L1SAP_IS_LINK_SACCH(trx_chan_desc[chan].link_id))
-		l1ts->chan_state[chan].lost_frames = 0;
-
 	/* forward primitive */
 	l1sap_up(l1ts->ts->trx, l1sap);
 
@@ -839,9 +825,6 @@ int _sched_compose_tch_ind(struct l1sched_ts *l1ts, uint32_t fn,
 	if (tch_len)
 		memcpy(msg->l2h, tch, tch_len);
 
-	if (l1ts->chan_state[chan].lost_frames)
-		l1ts->chan_state[chan].lost_frames--;
-
 	LOGL1S(DL1P, LOGL_DEBUG, l1ts, chan, l1sap->u.data.fn, "%s Rx -> RTP: %s\n",
 	       gsm_lchan_name(lchan), osmo_hexdump(msgb_l2(msg), msgb_l2len(msg)));
 	/* forward primitive */
@@ -865,9 +848,6 @@ int trx_sched_ph_data_req(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap
 		"PH-DATA.req: chan_nr=0x%02x link_id=0x%02x\n",
 		l1sap->u.data.chan_nr, l1sap->u.data.link_id);
 
-	OSMO_ASSERT(l1sap->oph.operation == PRIM_OP_REQUEST);
-	OSMO_ASSERT(l1sap->oph.msg);
-
 	/* ignore empty frame */
 	if (!l1sap->oph.msg->l2h || msgb_l2len(l1sap->oph.msg) == 0) {
 		msgb_free(l1sap->oph.msg);
@@ -891,9 +871,6 @@ int trx_sched_tch_req(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 	LOGL1S(DL1P, LOGL_DEBUG, l1ts, -1, l1sap->u.tch.fn,
 	       "TCH.req: chan_nr=0x%02x\n", l1sap->u.tch.chan_nr);
 
-	OSMO_ASSERT(l1sap->oph.operation == PRIM_OP_REQUEST);
-	OSMO_ASSERT(l1sap->oph.msg);
-
 	/* ignore empty frame */
 	if (!msgb_l2len(l1sap->oph.msg)) {
 		msgb_free(l1sap->oph.msg);
@@ -1013,12 +990,21 @@ static int rts_tchf_fn(const struct l1sched_ts *l1ts, const struct trx_dl_burst_
 	return rts_tch_common(l1ts, br, true);
 }
 
+/* FACCH/H channel mapping for Downlink (see 3GPP TS 45.002, table 1).
+ * This mapping is valid for both FACCH/H(0) and FACCH/H(1). */
+const uint8_t sched_tchh_dl_facch_map[26] = {
+	[4]  = 1, /* FACCH/H(0): B0(4,6,8,10,13,15) */
+	[5]  = 1, /* FACCH/H(1): B0(5,7,9,11,14,16) */
+	[13] = 1, /* FACCH/H(0): B1(13,15,17,19,21,23) */
+	[14] = 1, /* FACCH/H(1): B1(14,16,18,20,22,24) */
+	[21] = 1, /* FACCH/H(0): B2(21,23,0,2,4,6) */
+	[22] = 1, /* FACCH/H(1): B2(22,24,1,3,5,7) */
+};
 
 /* RTS for half rate traffic frame */
 static int rts_tchh_fn(const struct l1sched_ts *l1ts, const struct trx_dl_burst_req *br)
 {
-	/* the FN 4/5, 13/14, 21/22 defines that FACCH may be included. */
-	return rts_tch_common(l1ts, br, ((br->fn % 26) >> 2) & 1);
+	return rts_tch_common(l1ts, br, sched_tchh_dl_facch_map[br->fn % 26]);
 }
 
 /* set multiframe scheduler to given pchan */
@@ -1120,6 +1106,10 @@ int trx_sched_set_lchan(struct gsm_lchan *lchan, uint8_t chan_nr, uint8_t link_i
 			talloc_free(chan_state->ul_bursts);
 			chan_state->ul_bursts = NULL;
 		}
+		if (chan_state->ul_bursts_prev) {
+			talloc_free(chan_state->ul_bursts_prev);
+			chan_state->ul_bursts_prev = NULL;
+		}
 
 		if (active) {
 			/* Clean up everything */
@@ -1170,7 +1160,7 @@ int trx_sched_set_mode(struct gsm_bts_trx_ts *ts, uint8_t chan_nr, uint8_t rsl_c
 		 && trx_chan_desc[i].link_id == 0x00) {
 			struct l1sched_chan_state *chan_state = &l1ts->chan_state[i];
 
-			LOGP(DL1C, LOGL_NOTICE,
+			LOGP(DL1C, LOGL_INFO,
 			     "%s Set mode for %s (rsl_cmode=%u, tch_mode=%u, handover=%u)\n",
 			     gsm_ts_name(ts), trx_chan_desc[i].name,
 			     rsl_cmode, tch_mode, handover);
@@ -1189,8 +1179,8 @@ int trx_sched_set_mode(struct gsm_bts_trx_ts *ts, uint8_t chan_nr, uint8_t rsl_c
 				chan_state->dl_ft = initial_id;
 				chan_state->ul_cmr = initial_id;
 				chan_state->dl_cmr = initial_id;
-				chan_state->ber_sum = 0;
-				chan_state->ber_num = 0;
+				chan_state->lqual_cb_sum = 0;
+				chan_state->lqual_cb_num = 0;
 			}
 			rc = 0;
 		}
@@ -1237,7 +1227,7 @@ int trx_sched_set_cipher(struct gsm_lchan *lchan, uint8_t chan_nr, bool downlink
 			struct l1sched_ts *l1ts = lchan->ts->priv;
 			struct l1sched_chan_state *l1cs = &l1ts->chan_state[i];
 
-			LOGPLCHAN(lchan, DL1C, LOGL_NOTICE, "Set A5/%d %s for %s\n",
+			LOGPLCHAN(lchan, DL1C, LOGL_INFO, "Set A5/%d %s for %s\n",
 				  algo, (downlink) ? "downlink" : "uplink",
 				  trx_chan_desc[i].name);
 
@@ -1553,16 +1543,9 @@ int trx_sched_ul_burst(struct l1sched_ts *l1ts, struct trx_ul_burst_ind *bi)
 
 	/* handle NOPE indications */
 	if (bi->flags & TRX_BI_F_NOPE_IND) {
-		switch (bi->chan) {
-		case TRXC_PTCCH:
-		case TRXC_RACH:
-			/* For some logical channel types NOPE.ind is valueless. */
-			return 0;
-		default:
-			/* NOTE: Uplink burst handler must check bi->burst_len before
-			 * accessing bi->burst to avoid uninitialized memory access. */
-			return func(l1ts, bi);
-		}
+		/* NOTE: Uplink burst handler must check bi->burst_len before
+		 * accessing bi->burst to avoid uninitialized memory access. */
+		return func(l1ts, bi);
 	}
 
 	/* decrypt */
