@@ -57,6 +57,7 @@
 #include <osmo-bts/measurement.h>
 #include <osmo-bts/vty.h>
 #include <osmo-bts/l1sap.h>
+#include <osmo-bts/osmux.h>
 
 #define VTY_STR	"Configure the VTY\n"
 
@@ -140,6 +141,7 @@ int bts_vty_is_config_node(struct vty *vty, int node)
 	case BTS_NODE:
 	case PHY_NODE:
 	case PHY_INST_NODE:
+	case OSMUX_NODE:
 		return 1;
 	default:
 		return 0;
@@ -147,7 +149,8 @@ int bts_vty_is_config_node(struct vty *vty, int node)
 }
 
 static const char osmobts_copyright[] =
-	"Copyright (C) 2010, 2011 by Harald Welte, Andreas Eversberg and On-Waves\r\n"
+	"Copyright (C) 2010-2011 by Harald Welte, Andreas Eversberg and On-Waves\r\n"
+	"Copyright (C) 2011-2022 by sysmocom - s.f.m.c. GmbH\r\n"
 	"License AGPLv3+: GNU AGPL version 3 or later <http://gnu.org/licenses/agpl-3.0.html>\r\n"
 	"This is free software: you are free to change and redistribute it.\r\n"
 	 "There is NO WARRANTY, to the extent permitted by law.\r\n";
@@ -189,6 +192,12 @@ static struct cmd_node trx_node = {
 	1,
 };
 
+static struct cmd_node osmux_node = {
+	OSMUX_NODE,
+	"%s(osmux)# ",
+	1,
+};
+
 gDEFUN(cfg_bts_auto_band, cfg_bts_auto_band_cmd,
 	"auto-band",
 	"Automatically select band for ARFCN based on configured band\n")
@@ -209,6 +218,90 @@ gDEFUN(cfg_bts_no_auto_band, cfg_bts_no_auto_band_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN_ATTR(cfg_bts_osmux, cfg_bts_osmux_cmd,
+	   "osmux",
+	   "Configure Osmux\n",
+	   CMD_ATTR_IMMEDIATE)
+{
+	vty->node = OSMUX_NODE;
+	return CMD_SUCCESS;
+}
+
+DEFUN_ATTR(cfg_bts_osmux_use, cfg_bts_osmux_use_cmd,
+	   "use (off|on|only)",
+	   "Configure Osmux usage\n"
+	   "Never use Osmux\n"
+	   "Use Osmux if requested by BSC (default)\n"
+	   "Always use Osmux, reject non-Osmux BSC requests\n",
+	   CMD_ATTR_IMMEDIATE)
+{
+	struct gsm_bts *bts = vty->index;
+	if (strcmp(argv[0], "off") == 0)
+		bts->osmux.use = OSMUX_USAGE_OFF;
+	else if (strcmp(argv[0], "on") == 0)
+		bts->osmux.use = OSMUX_USAGE_ON;
+	else if (strcmp(argv[0], "only") == 0)
+		bts->osmux.use = OSMUX_USAGE_ONLY;
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_bts_osmux_ip,
+      cfg_bts_osmux_ip_cmd,
+      "local-ip " VTY_IPV46_CMD,
+      IP_STR
+      "IPv4 Address to bind to\n"
+      "IPv6 Address to bind to\n")
+{
+	struct gsm_bts *bts = vty->index;
+	osmo_talloc_replace_string(bts, &bts->osmux.local_addr, argv[0]);
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_bts_osmux_port,
+      cfg_bts_osmux_port_cmd,
+      "local-port <1-65535>",
+      "Osmux port\n" "UDP port\n")
+{
+	struct gsm_bts *bts = vty->index;
+	bts->osmux.local_port = atoi(argv[0]);
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_bts_osmux_batch_factor,
+      cfg_bts_osmux_batch_factor_cmd,
+      "batch-factor <1-8>",
+      "Batching factor\n" "Number of messages in the batch\n")
+{
+	struct gsm_bts *bts = vty->index;
+	bts->osmux.batch_factor = atoi(argv[0]);
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_bts_osmux_batch_size,
+      cfg_bts_osmux_batch_size_cmd,
+      "batch-size <1-65535>",
+      "Batch size\n" "Batch size in bytes\n")
+{
+	struct gsm_bts *bts = vty->index;
+	bts->osmux.batch_size = atoi(argv[0]);
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_bts_osmux_dummy_padding,
+      cfg_bts_osmux_dummy_padding_cmd,
+      "dummy-padding (on|off)",
+      "Dummy padding\n"
+      "Enable dummy padding\n"
+      "Disable dummy padding (default)\n")
+{
+	struct gsm_bts *bts = vty->index;
+	if (strcmp(argv[0], "on") == 0)
+		bts->osmux.dummy_padding = true;
+	else if (strcmp(argv[0], "off") == 0)
+		bts->osmux.dummy_padding = false;
+	return CMD_SUCCESS;
+}
+
 DEFUN_ATTR(cfg_bts_trx, cfg_bts_trx_cmd,
 	   "trx <0-254>",
 	   "Select a TRX to configure\n" "TRX number\n",
@@ -278,6 +371,29 @@ static void config_write_dpc_params(struct vty *vty, const char *prefix,
 	}
 }
 
+static void config_write_osmux(struct vty *vty, const char *prefix, const struct gsm_bts *bts)
+{
+	vty_out(vty, "%sosmux%s", prefix, VTY_NEWLINE);
+	vty_out(vty, "%s use ", prefix);
+	switch (bts->osmux.use) {
+	case OSMUX_USAGE_ON:
+		vty_out(vty, "on%s", VTY_NEWLINE);
+		break;
+	case OSMUX_USAGE_ONLY:
+		vty_out(vty, "only%s", VTY_NEWLINE);
+		break;
+	case OSMUX_USAGE_OFF:
+	default:
+		vty_out(vty, "off%s", VTY_NEWLINE);
+		break;
+	}
+	vty_out(vty, "%s local-ip %s%s", prefix, bts->osmux.local_addr, VTY_NEWLINE);
+	vty_out(vty, "%s local-port %u%s", prefix, bts->osmux.local_port, VTY_NEWLINE);
+	vty_out(vty, "%s batch-factor %d%s", prefix, bts->osmux.batch_factor, VTY_NEWLINE);
+	vty_out(vty, "%s batch-size %u%s", prefix, bts->osmux.batch_size, VTY_NEWLINE);
+	vty_out(vty, "%s dummy-padding %s%s", prefix, bts->osmux.dummy_padding ? "on" : "off", VTY_NEWLINE);
+}
+
 static void config_write_bts_single(struct vty *vty, const struct gsm_bts *bts)
 {
 	const struct gsm_bts_trx *trx;
@@ -351,6 +467,8 @@ static void config_write_bts_single(struct vty *vty, const struct gsm_bts *bts)
 	vty_out(vty, " smscb queue-target-length %d%s", bts->smscb_queue_tgt_len, VTY_NEWLINE);
 	vty_out(vty, " smscb queue-hysteresis %d%s", bts->smscb_queue_hyst, VTY_NEWLINE);
 
+	config_write_osmux(vty, " ", bts);
+
 	bts_model_config_write_bts(vty, bts);
 
 	llist_for_each_entry(trx, &bts->trx_list, list) {
@@ -1329,7 +1447,7 @@ DEFUN_HIDDEN(radio_link_timeout, radio_link_timeout_cmd, "bts <0-0> radio-link-t
 
 DEFUN(bts_c0_power_red,
       bts_c0_power_red_cmd,
-      "bts <0-255> c0-power-red <0-6>",
+      "bts <0-0> c0-power-red <0-6>",
       "BTS Specific Commands\n" BTS_NR_STR
       "BCCH carrier power reduction operation\n"
       "Power reduction value (in dB, even numbers only)\n")
@@ -1822,17 +1940,23 @@ static void lchan_dump_full_vty(struct vty *vty, const struct gsm_lchan *lchan)
 
 	if (lchan->abis_ip.bound_ip) {
 		ia.s_addr = htonl(lchan->abis_ip.bound_ip);
-		vty_out(vty, "  Bound IP: %s Port %u RTP_TYPE2=%u CONN_ID=%u%s",
+		vty_out(vty, "  Bound IP: %s Port %u CONN_ID=%u",
 			inet_ntoa(ia), lchan->abis_ip.bound_port,
-			lchan->abis_ip.rtp_payload2, lchan->abis_ip.conn_id,
-			VTY_NEWLINE);
+			lchan->abis_ip.conn_id);
+		if (lchan->abis_ip.osmux.use)
+			vty_out(vty, " Osmux_CID=%u%s", lchan->abis_ip.osmux.local_cid, VTY_NEWLINE);
+		else
+			vty_out(vty, " RTP_TYPE2=%u%s", lchan->abis_ip.rtp_payload2, VTY_NEWLINE);
 	}
 	if (lchan->abis_ip.connect_ip) {
 		ia.s_addr = htonl(lchan->abis_ip.connect_ip);
-		vty_out(vty, "  Conn. IP: %s Port %u RTP_TYPE=%u SPEECH_MODE=0x%02u%s",
+		vty_out(vty, "  Conn. IP: %s Port %u SPEECH_MODE=0x%02x",
 			inet_ntoa(ia), lchan->abis_ip.connect_port,
-			lchan->abis_ip.rtp_payload, lchan->abis_ip.speech_mode,
-			VTY_NEWLINE);
+			lchan->abis_ip.speech_mode);
+		if (lchan->abis_ip.osmux.use)
+			vty_out(vty, " Osmux_CID=%u%s", lchan->abis_ip.osmux.remote_cid, VTY_NEWLINE);
+		else
+			vty_out(vty, " RTP_TYPE=%u%s", lchan->abis_ip.rtp_payload, VTY_NEWLINE);
 	}
 #define LAPDM_ESTABLISHED(link, sapi_idx) \
 		(link).datalink[sapi_idx].dl.state == LAPD_STATE_MF_EST
@@ -2146,7 +2270,7 @@ DEFUN(cfg_bts_gsmtap_sapi, cfg_bts_gsmtap_sapi_cmd,
 	return CMD_SUCCESS;
 }
 
-DEFUN(cfg_trx_no_gsmtap_sapi, cfg_bts_no_gsmtap_sapi_cmd,
+DEFUN(cfg_bts_no_gsmtap_sapi, cfg_bts_no_gsmtap_sapi_cmd,
 	"HIDDEN", "HIDDEN")
 {
 	struct gsm_bts *bts = vty->index;
@@ -2532,6 +2656,17 @@ int bts_vty_init(void *ctx)
 	install_element(BTS_NODE, &cfg_bts_gsmtap_sapi_cmd);
 	install_element(BTS_NODE, &cfg_bts_no_gsmtap_sapi_cmd);
 
+	/* Osmux Node */
+	install_element(BTS_NODE, &cfg_bts_osmux_cmd);
+	install_node(&osmux_node, config_write_dummy);
+
+	install_element(OSMUX_NODE, &cfg_bts_osmux_use_cmd);
+	install_element(OSMUX_NODE, &cfg_bts_osmux_ip_cmd);
+	install_element(OSMUX_NODE, &cfg_bts_osmux_port_cmd);
+	install_element(OSMUX_NODE, &cfg_bts_osmux_batch_factor_cmd);
+	install_element(OSMUX_NODE, &cfg_bts_osmux_batch_size_cmd);
+	install_element(OSMUX_NODE, &cfg_bts_osmux_dummy_padding_cmd);
+
 	/* add and link to TRX config node */
 	install_element(BTS_NODE, &cfg_bts_trx_cmd);
 	install_node(&trx_node, config_write_dummy);
