@@ -459,7 +459,7 @@ static int mph_send_activate_req(struct gsm_lchan *lchan, struct sapi_cmd *cmd)
 	lac->LchId.bySAPI = cmd->sapi;
 	lac->LchId.byDirection = cmd->dir;
 
-	lac->Config.byTimingAdvance = lchan->rqd_ta;
+	lac->Config.byTimingAdvance = lchan->ta_ctrl.current;
 	lac->Config.byBSIC = lchan->ts->trx->bts->bsic;
 	if ((rc = lchan2lch_par(lchan, &lac->Config)) != 0) {
 		talloc_free(msg);
@@ -1107,9 +1107,6 @@ int lchan_activate(struct gsm_lchan *lchan)
 		}
 		enqueue_sapi_act_cmd(lchan, sapi, dir);
 	}
-
-	lchan_init_lapdm(lchan);
-
 	return 0;
 }
 
@@ -1119,13 +1116,6 @@ int l1if_rsl_chan_act(struct gsm_lchan *lchan)
 	return 0;
 }
 
-#define talloc_replace(dst, ctx, src)			\
-	do {						\
-		if (dst)				\
-			talloc_free(dst);		\
-		dst = talloc_strdup(ctx, (const char *) src);	\
-	} while (0)
-
 static int app_info_sys_compl_cb(struct octphy_hdl *fl1h, struct msgb *resp, void *data)
 {
 	tOCTVC1_MAIN_MSG_APPLICATION_INFO_SYSTEM_RSP *aisr =
@@ -1145,8 +1135,10 @@ static int app_info_sys_compl_cb(struct octphy_hdl *fl1h, struct msgb *resp, voi
 	LOGP(DL1C, LOGL_INFO, "Note: compiled without multi-trx support.\n");
 #endif
 
-	talloc_replace(fl1h->info.system.platform, fl1h, aisr->szPlatform);
-	talloc_replace(fl1h->info.system.version, fl1h, aisr->szVersion);
+	osmo_talloc_replace_string(fl1h, &fl1h->info.system.platform,
+				   (const char *) aisr->szPlatform);
+	osmo_talloc_replace_string(fl1h, &fl1h->info.system.version,
+				   (const char *) aisr->szVersion);
 
 	msgb_free(resp);
 
@@ -1210,9 +1202,12 @@ static int app_info_compl_cb(struct octphy_hdl *fl1h, struct msgb *resp,
 		}
 	}
 
-	talloc_replace(fl1h->info.app.name, fl1h, air->szName);
-	talloc_replace(fl1h->info.app.description, fl1h, air->szDescription);
-	talloc_replace(fl1h->info.app.version, fl1h, air->szVersion);
+	osmo_talloc_replace_string(fl1h, &fl1h->info.app.name,
+				   (const char *) air->szName);
+	osmo_talloc_replace_string(fl1h, &fl1h->info.app.description,
+				   (const char *) air->szDescription);
+	osmo_talloc_replace_string(fl1h, &fl1h->info.app.version,
+				   (const char *) air->szVersion);
 	OSMO_ASSERT(strlen(ver_hdr) < sizeof(pinst->version));
 	osmo_strlcpy(pinst->version, ver_hdr, strlen(ver_hdr));
 
@@ -1352,7 +1347,7 @@ int l1if_trx_open(struct gsm_bts_trx *trx)
 	}
 	oc->Config.usBcchArfcn = trx->bts->c0->arfcn;
 #endif
-	oc->Config.usTsc = trx->bts->bsic & 0x7;
+	oc->Config.usTsc = BTS_TSC(trx->bts);
 	oc->RfConfig.ulRxGainDb = plink->u.octphy.rx_gain_db;
 	/* FIXME: compute this based on nominal transmit power, etc. */
 	if (plink->u.octphy.tx_atten_flag) {
@@ -1750,13 +1745,36 @@ int bts_model_check_oml(struct gsm_bts *bts, uint8_t msg_type,
 int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg,
 			struct tlv_parsed *new_attr, int kind, void *obj)
 {
-	if (kind == NM_OC_RADIO_CARRIER) {
-		struct gsm_bts_trx *trx = obj;
-		/*struct octphy_hdl *fl1h = trx_octphy_hdl(trx); */
+	struct abis_om_fom_hdr *foh = msgb_l3(msg);
+	struct gsm_abis_mo *mo = gsm_objclass2mo(bts, foh->obj_class, &foh->obj_inst);
+	struct nm_fsm_ev_setattr_data ev_data = {
+		.msg = msg,
+		.cause = 0,
+	};
+	int rc;
+	struct gsm_bts_trx *trx;
 
-		power_ramp_start(trx, get_p_target_mdBm(trx, 0), 0, NULL);
+	/* TODO: NM Object without FSM: */
+	switch (foh->obj_class) {
+	case NM_OC_GPRS_NSE:
+	case NM_OC_GPRS_CELL:
+	case NM_OC_GPRS_NSVC:
+		return oml_fom_ack_nack(ev_data.msg, ev_data.cause);
 	}
-	return oml_fom_ack_nack(msg, 0);
+
+	switch (foh->msg_type) {
+	case NM_MT_SET_RADIO_ATTR:
+		trx = obj;
+		/*struct octphy_hdl *fl1h = trx_octphy_hdl(trx); */
+		power_ramp_start(trx, get_p_target_mdBm(trx, 0), 0, NULL);
+		break;
+	}
+
+	rc = osmo_fsm_inst_dispatch(mo->fi,
+				    ev_data.cause == 0 ? NM_EV_SETATTR_ACK : NM_EV_SETATTR_NACK,
+				    &ev_data);
+	/* msgb ownsership is transferred to FSM if it received ev: */
+	return rc == 0 ? 1 : 0;
 }
 
 
@@ -1790,7 +1808,7 @@ int bts_model_opstart(struct gsm_bts *bts, struct gsm_abis_mo *mo, void *obj)
 	case NM_OC_GPRS_NSE:
 	case NM_OC_GPRS_CELL:
 	case NM_OC_GPRS_NSVC:
-		oml_mo_state_chg(mo, NM_OPSTATE_ENABLED, -1);
+		oml_mo_state_chg(mo, NM_OPSTATE_ENABLED, -1, -1);
 		rc = oml_mo_opstart_ack(mo);
 		break;
 	default:
@@ -1834,7 +1852,7 @@ void bts_model_ts_connect(struct gsm_bts_trx_ts *ts,
 {
 	int rc;
 	if (as_pchan == GSM_PCHAN_TCH_F_PDCH
-	    || as_pchan == GSM_PCHAN_TCH_F_TCH_H_PDCH) {
+	    || as_pchan == GSM_PCHAN_OSMO_DYN) {
 		LOGP(DL1C, LOGL_ERROR,
 		     "%s Requested TS connect as %s,"
 		     " expected a specific pchan instead\n",
