@@ -195,7 +195,7 @@ void bts_model_abis_close(struct gsm_bts *bts)
 {
 	/* Go into shutdown state deactivating transceivers until Abis link
 	 * becomes up again */
-	bts_shutdown_ext(bts, "Abis close", false);
+	bts_shutdown_ext(bts, "Abis close", false, true);
 }
 
 int bts_model_adjst_ms_pwr(struct gsm_lchan *lchan)
@@ -326,8 +326,8 @@ static uint8_t trx_set_ts(struct gsm_bts_trx_ts *ts)
 /* enable ciphering */
 static int l1if_set_ciphering(struct gsm_lchan *lchan, uint8_t chan_nr, int downlink)
 {
-	/* ciphering already enabled in both directions */
-	if (lchan->ciph_state == LCHAN_CIPH_RXTX_CONF)
+	/* ignore the request when the channel is not active */
+	if (lchan->state != LCHAN_S_ACTIVE)
 		return -EINVAL;
 
 	if (!downlink) {
@@ -428,10 +428,10 @@ int bts_model_l1sap_down(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 				trx_sched_set_mode(lchan->ts, chan_nr,
 					lchan->rsl_cmode, lchan->tch_mode,
 					lchan->tch.amr_mr.num_modes,
-					lchan->tch.amr_mr.bts_mode[0].mode,
-					lchan->tch.amr_mr.bts_mode[1].mode,
-					lchan->tch.amr_mr.bts_mode[2].mode,
-					lchan->tch.amr_mr.bts_mode[3].mode,
+					lchan->tch.amr_mr.mode[0].mode,
+					lchan->tch.amr_mr.mode[1].mode,
+					lchan->tch.amr_mr.mode[2].mode,
+					lchan->tch.amr_mr.mode[3].mode,
 					amr_get_initial_mode(lchan),
 					(lchan->ho.active == 1));
 				/* set lchan active */
@@ -457,12 +457,19 @@ int bts_model_l1sap_down(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 				trx_sched_set_mode(lchan->ts, chan_nr,
 					lchan->rsl_cmode, lchan->tch_mode,
 					lchan->tch.amr_mr.num_modes,
-					lchan->tch.amr_mr.bts_mode[0].mode,
-					lchan->tch.amr_mr.bts_mode[1].mode,
-					lchan->tch.amr_mr.bts_mode[2].mode,
-					lchan->tch.amr_mr.bts_mode[3].mode,
+					lchan->tch.amr_mr.mode[0].mode,
+					lchan->tch.amr_mr.mode[1].mode,
+					lchan->tch.amr_mr.mode[2].mode,
+					lchan->tch.amr_mr.mode[3].mode,
 					amr_get_initial_mode(lchan),
 					0);
+				/* update ciphering params */
+				l1if_set_ciphering(lchan, chan_nr, 0);
+				l1if_set_ciphering(lchan, chan_nr, 1);
+				if (lchan->encr.alg_id)
+					lchan->ciph_state = LCHAN_CIPH_RXTX_CONF;
+				else
+					lchan->ciph_state = LCHAN_CIPH_NONE;
 				break;
 			}
 			/* here, type == PRIM_INFO_DEACTIVATE */
