@@ -35,6 +35,8 @@
 #include <osmocom/coding/gsm0503_coding.h>
 #include <osmocom/coding/gsm0503_amr_dtx.h>
 
+#include <osmocom/netif/amr.h>
+
 #include <osmo-bts/bts.h>
 #include <osmo-bts/l1sap.h>
 #include <osmo-bts/logging.h>
@@ -43,7 +45,30 @@
 #include <osmo-bts/msg_utils.h>
 
 #include <sched_utils.h>
-#include <loops.h>
+#include <amr_loop.h>
+
+/* 3GPP TS 45.009, table 3.2.1.3-{1,3}: AMR on Uplink TCH/F.
+ *
+ * +---+---+---+---+---+---+---+---+
+ * | a | b | c | d | e | f | g | h |  Burst 'a' received first
+ * +---+---+---+---+---+---+---+---+
+ *  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^   Speech/FACCH frame  (bursts 'a' .. 'h')
+ *
+ * TDMA frame number of burst 'h' is always used as the table index. */
+static const uint8_t sched_tchf_ul_amr_cmi_map[26] = {
+	[7]  = 1, /* TCH/F: a=0  / h=7 */
+	[16] = 1, /* TCH/F: a=8  / h=16 */
+	[24] = 1, /* TCH/F: a=17 / h=24 */
+};
+
+/* TDMA frame number of burst 'a' should be used as the table index. */
+static const uint8_t sched_tchf_dl_amr_cmi_map[26] = {
+	[4]  = 1, /* TCH/F: a=4 */
+	[13] = 1, /* TCH/F: a=13 */
+	[21] = 1, /* TCH/F: a=21 */
+};
+
+extern const uint8_t sched_tchh_dl_amr_cmi_map[26];
 
 /*! \brief a single TCH/F burst was received by the PHY, process it */
 int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
@@ -55,7 +80,7 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 	uint8_t rsl_cmode = chan_state->rsl_cmode;
 	uint8_t tch_mode = chan_state->tch_mode;
 	uint8_t tch_data[128]; /* just to be safe */
-	enum sched_meas_avg_mode meas_avg_mode = SCHED_MEAS_AVG_M_OCTO;
+	enum sched_meas_avg_mode meas_avg_mode = SCHED_MEAS_AVG_M_S8N8;
 	struct l1sched_meas_set meas_avg;
 	int rc, amr = 0;
 	int n_errors = 0;
@@ -76,15 +101,16 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 
 	/* allocate burst memory, if not already */
 	if (!*bursts_p) {
-		*bursts_p = talloc_zero_size(tall_bts_ctx, 928);
+		*bursts_p = talloc_zero_size(l1ts, 928);
 		if (!*bursts_p)
 			return -ENOMEM;
 	}
 
-	/* clear burst */
+	/* shift the buffer by 4 bursts leftwards */
 	if (bi->bid == 0) {
+		memcpy(*bursts_p, *bursts_p + 464, 464);
 		memset(*bursts_p + 464, 0, 464);
-		*mask = 0x0;
+		*mask = *mask << 4;
 	}
 
 	/* update mask */
@@ -105,21 +131,21 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 	if (bi->bid != 3)
 		return 0;
 
-	/* check for complete set of bursts */
-	if ((*mask & 0xf) != 0xf) {
-		LOGL1SB(DL1P, LOGL_NOTICE, l1ts, bi,
-			"Received incomplete frame (%u/%u)\n",
-			bi->fn % l1ts->mf_period, l1ts->mf_period);
+	/* fill up the burst buffer so that we have 8 bursts in there */
+	if (OSMO_UNLIKELY((*mask & 0xff) != 0xff)) {
+		LOGL1SB(DL1P, LOGL_DEBUG, l1ts, bi,
+			"UL burst buffer is not filled up: mask=0x%02x != 0xff\n",
+			*mask);
+		return 0; /* TODO: send BFI */
 	}
-	*mask = 0x0;
 
 	/* decode
 	 * also shift buffer by 4 bursts for interleaving */
-	switch ((rsl_cmode != RSL_CMOD_SPD_SPEECH) ? GSM48_CMODE_SPEECH_V1
-								: tch_mode) {
+	switch (tch_mode) {
+	case GSM48_CMODE_SIGN:
 	case GSM48_CMODE_SPEECH_V1: /* FR */
 		rc = gsm0503_tch_fr_decode(tch_data, *bursts_p, 1, 0, &n_errors, &n_bits_total);
-		if (rc >= 0)
+		if (rc == GSM_FR_BYTES) /* only for valid *speech* frames */
 			lchan_set_marker(osmo_fr_check_sid(tch_data, rc), lchan); /* DTXu */
 		break;
 	case GSM48_CMODE_SPEECH_EFR: /* EFR */
@@ -130,8 +156,7 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		 * the first FN 4,13,21 defines that CMR is included in frame.
 		 * NOTE: A frame ends 7 FN after start.
 		 */
-		fn_begin = gsm0502_fn_remap(bi->fn, FN_REMAP_TCH_F);
-		amr_is_cmr = !ul_amr_fn_is_cmi(fn_begin);
+		amr_is_cmr = !sched_tchf_ul_amr_cmi_map[bi->fn % 26];
 
 		/* The AFS_ONSET frame itself does not result into an RTP frame
 		 * since it only contains a recognition pattern that marks the
@@ -141,11 +166,13 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		if (chan_state->amr_last_dtx == AFS_ONSET)
 			lchan_set_marker(false, lchan);
 
-		/* we store tch_data + 2 header bytes, the amr variable set to
-		 * 2 will allow us to skip the first 2 bytes in case we did
-		 * receive an FACCH frame instead of a voice frame (we do not
-		 * know this before we actually decode the frame) */
-		amr = 2;
+		/* Store AMR payload in tch-data with an offset of 2 bytes, so
+		 * that we can easily prepend/fill the RTP AMR header (struct
+		 * amr_hdr) with osmo_amr_rtp_enc() later on. The amr variable
+		 * is used far below to account for the decoded offset in case
+		 * we receive an FACCH frame instead of a voice frame (we
+		 * do not know this before we actually decode the frame) */
+		amr = sizeof(struct amr_hdr);
 		rc = gsm0503_tch_afs_decode_dtx(tch_data + amr, *bursts_p,
 			amr_is_cmr, chan_state->codec, chan_state->codecs, &chan_state->ul_ft,
 			&chan_state->ul_cmr, &n_errors, &n_bits_total, &chan_state->amr_last_dtx);
@@ -153,8 +180,10 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		/* Tag all frames that are not regular AMR voice frames as
 		 * SUB-Frames */
 		if (chan_state->amr_last_dtx != AMR_OTHER) {
-			LOGL1SB(DL1P, LOGL_DEBUG, l1ts, bi, "Received AMR SID frame: %s\n",
-			       gsm0503_amr_dtx_frame_name(chan_state->amr_last_dtx));
+			LOGL1SB(DL1P, LOGL_DEBUG, l1ts, bi,
+				"Received AMR DTX frame (rc=%d, BER %d/%d): %s\n",
+				rc, n_errors, n_bits_total,
+				gsm0503_amr_dtx_frame_name(chan_state->amr_last_dtx));
 			is_sub = 1;
 		}
 
@@ -175,16 +204,14 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		switch (chan_state->amr_last_dtx) {
 		case AFS_SID_FIRST:
 		case AFS_SID_UPDATE_CN:
-			meas_avg_mode = SCHED_MEAS_AVG_M8_FIRST_QUAD;
+			meas_avg_mode = SCHED_MEAS_AVG_M_S8N4;
 			break;
 		case AFS_SID_UPDATE:
 		case AFS_ONSET:
-			meas_avg_mode = SCHED_MEAS_AVG_M_QUAD;
+			meas_avg_mode = SCHED_MEAS_AVG_M_S4N4;
 			break;
 		}
 
-		if (rc)
-			trx_loop_amr_input(chan_state, n_errors, n_bits_total);
 		/* only good speech frames get rtp header */
 		if (rc != GSM_MACBLOCK_LEN && rc >= 4) {
 			if (chan_state->amr_last_dtx == AMR_OTHER) {
@@ -205,10 +232,14 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 			tch_mode);
 		return -EINVAL;
 	}
-	memcpy(*bursts_p, *bursts_p + 464, 464);
 
 	/* average measurements of the last N (depends on mode) bursts */
 	trx_sched_meas_avg(chan_state, &meas_avg, meas_avg_mode);
+	/* meas_avg.fn now contains TDMA frame number of the first burst */
+	fn_begin = meas_avg.fn;
+
+	if (tch_mode == GSM48_CMODE_SPEECH_AMR)
+		trx_loop_amr_input(chan_state, &meas_avg);
 
 	/* Check if the frame is bad */
 	if (rc < 0) {
@@ -231,7 +262,6 @@ int rx_tchf_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 
 	/* FACCH */
 	if (rc == GSM_MACBLOCK_LEN) {
-		fn_begin = gsm0502_fn_remap(bi->fn, FN_REMAP_FACCH_F);
 		_sched_compose_ph_data_ind(l1ts, fn_begin, bi->chan,
 			tch_data + amr, GSM_MACBLOCK_LEN,
 			meas_avg.rssi, meas_avg.toa256,
@@ -285,7 +315,7 @@ bfi:
 					       "not sending BFI\n", rc);
 					return -EINVAL;
 				}
-				memset(tch_data + 2, 0, rc - 2);
+				memset(tch_data + sizeof(struct amr_hdr), 0, rc - sizeof(struct amr_hdr));
 				break;
 			default:
 				LOGL1SB(DL1P, LOGL_ERROR, l1ts, bi,
@@ -300,19 +330,14 @@ bfi:
 
 	/* TCH or BFI */
 compose_l1sap:
-	fn_begin = gsm0502_fn_remap(bi->fn, FN_REMAP_TCH_F);
 	return _sched_compose_tch_ind(l1ts, fn_begin, bi->chan, tch_data, rc,
-				      /* FIXME: what should we use for BFI here? */
-				      bfi_flag ? bi->toa256 : meas_avg.toa256, ber10k,
-				      bfi_flag ? bi->rssi : meas_avg.rssi,
-				      bfi_flag ? bi->ci_cb : meas_avg.ci_cb,
-				      is_sub);
+				      meas_avg.toa256, ber10k, meas_avg.rssi,
+				      meas_avg.ci_cb, is_sub);
 }
 
 /* common section for generation of TCH bursts (TCH/H and TCH/F).
  * FIXME: this function is over-complicated, refactor / get rid of it. */
-void tx_tch_common(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br,
-		   struct msgb **_msg_tch, struct msgb **_msg_facch)
+struct msgb *tch_dl_dequeue(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 {
 	struct msgb *msg1, *msg2, *msg_tch = NULL, *msg_facch = NULL;
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[br->chan];
@@ -320,58 +345,6 @@ void tx_tch_common(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br,
 	uint8_t tch_mode = chan_state->tch_mode;
 	struct osmo_phsap_prim *l1sap;
 
-	/* handle loss detection of received TCH frames */
-	if (rsl_cmode == RSL_CMOD_SPD_SPEECH
-	    && ++(chan_state->lost_frames) > 5) {
-		uint8_t tch_data[GSM_FR_BYTES];
-		int len;
-
-		LOGL1SB(DL1P, LOGL_NOTICE, l1ts, br, "Missing TCH bursts detected, sending BFI\n");
-
-		/* indicate bad frame */
-		switch (tch_mode) {
-		case GSM48_CMODE_SPEECH_V1: /* FR / HR */
-			if (br->chan != TRXC_TCHF) { /* HR */
-				tch_data[0] = 0x70; /* F = 0, FT = 111 */
-				memset(tch_data + 1, 0, 14);
-				len = 15;
-				break;
-			}
-			memset(tch_data, 0, GSM_FR_BYTES);
-			len = GSM_FR_BYTES;
-			break;
-		case GSM48_CMODE_SPEECH_EFR: /* EFR */
-			if (br->chan != TRXC_TCHF)
-				goto inval_mode1;
-			memset(tch_data, 0, GSM_EFR_BYTES);
-			len = GSM_EFR_BYTES;
-			break;
-		case GSM48_CMODE_SPEECH_AMR: /* AMR */
-			len = osmo_amr_rtp_enc(tch_data,
-				chan_state->codec[chan_state->dl_cmr],
-				chan_state->codec[chan_state->dl_ft], AMR_BAD);
-			if (len < 2) {
-				LOGL1SB(DL1P, LOGL_ERROR, l1ts, br,
-					"Failed to encode AMR_BAD frame (rc=%d), "
-					"not sending BFI\n", len);
-				return;
-			}
-			memset(tch_data + 2, 0, len - 2);
-			break;
-		default:
-inval_mode1:
-			LOGL1SB(DL1P, LOGL_ERROR, l1ts, br, "TCH mode invalid, please fix!\n");
-			len = 0;
-		}
-
-		if (len) {
-			/* Note: RSSI/ToA256 is set to 0 to indicate to the higher
-			 * layers that this is a faked tch_ind */
-			_sched_compose_tch_ind(l1ts, br->fn, br->chan,
-					       tch_data, len, 0, 10000, 0, 0, 0);
-		}
-	}
-
 	/* get frame and unlink from queue */
 	msg1 = _sched_dequeue_prim(l1ts, br);
 	msg2 = _sched_dequeue_prim(l1ts, br);
@@ -409,15 +382,26 @@ inval_mode1:
 		msg_facch = NULL;
 	}
 
+	/* prioritize FACCH over speech */
+	if (msg_facch) {
+		/* Unlike SACCH, FACCH has no dedicated slots on the multiframe layout.
+		 * It's multiplexed together with TCH (speech or data) frames basically
+		 * by replacing (stealing) them.  This is common for both TCH/F and
+		 * TCH/H, with the only difference that FACCH/H steals two TCH frames
+		 * (not just one) due to a longer interleaving period. */
+		msgb_free(msg_tch);
+		return msg_facch;
+	}
+
 	/* check validity of message, get AMR ft and cmr */
-	if (!msg_facch && msg_tch) {
+	if (msg_tch) {
 		int len;
 		uint8_t cmr_codec;
-		int cmr, ft, i;
+		int ft, i;
 		enum osmo_amr_type ft_codec;
 		enum osmo_amr_quality bfi;
 		int8_t sti, cmi;
-		bool amr_is_cmr = !dl_amr_fn_is_cmi(br->fn);
+		bool amr_is_cmr;
 
 		if (rsl_cmode != RSL_CMOD_SPD_SPEECH) {
 			LOGL1SB(DL1P, LOGL_NOTICE, l1ts, br, "Dropping speech frame, "
@@ -428,7 +412,7 @@ inval_mode1:
 		switch (tch_mode) {
 		case GSM48_CMODE_SPEECH_V1: /* FR / HR */
 			if (br->chan != TRXC_TCHF) /* HR */
-				len = 15;
+				len = GSM_HR_BYTES + 1; /* RFC 5993 */
 			else
 				len = GSM_FR_BYTES;
 			break;
@@ -441,27 +425,24 @@ inval_mode1:
 			len = osmo_amr_rtp_dec(msg_tch->l2h, msgb_l2len(msg_tch),
 					       &cmr_codec, &cmi, &ft_codec,
 					       &bfi, &sti);
-			cmr = -1;
+			if (len < 0) {
+				LOGL1SB(DL1P, LOGL_ERROR, l1ts, br, "Cannot send invalid AMR payload\n");
+				goto free_bad_msg;
+			}
 			ft = -1;
 			for (i = 0; i < chan_state->codecs; i++) {
-				if (chan_state->codec[i] == cmr_codec)
-					cmr = i;
 				if (chan_state->codec[i] == ft_codec)
 					ft = i;
 			}
-			if (cmr >= 0) { /* new request */
-				chan_state->dl_cmr = cmr;
-				/* disable AMR loop */
-				trx_loop_amr_set(chan_state, 0);
-			} else {
-				/* enable AMR loop */
-				trx_loop_amr_set(chan_state, 1);
-			}
 			if (ft < 0) {
 				LOGL1SB(DL1P, LOGL_ERROR, l1ts, br,
 					"Codec (FT = %d) of RTP frame not in list\n", ft_codec);
 				goto free_bad_msg;
 			}
+			if (br->chan == TRXC_TCHF)
+				amr_is_cmr = !sched_tchf_dl_amr_cmi_map[br->fn % 26];
+			else /* TRXC_TCHH_0 or TRXC_TCHH_1 */
+				amr_is_cmr = !sched_tchh_dl_amr_cmi_map[br->fn % 26];
 			if (amr_is_cmr && chan_state->dl_ft != ft) {
 				LOGL1SB(DL1P, LOGL_NOTICE, l1ts, br, "Codec (FT = %d) "
 					" of RTP cannot be changed now, but in next frame\n", ft_codec);
@@ -478,10 +459,6 @@ inval_mode2:
 			LOGL1SB(DL1P, LOGL_ERROR, l1ts, br, "TCH mode invalid, please fix!\n");
 			goto free_bad_msg;
 		}
-		if (len < 0) {
-			LOGL1SB(DL1P, LOGL_ERROR, l1ts, br, "Cannot send invalid AMR payload\n");
-			goto free_bad_msg;
-		}
 		if (msgb_l2len(msg_tch) != len) {
 			LOGL1SB(DL1P, LOGL_ERROR, l1ts, br, "Cannot send payload with "
 				"invalid length! (expecting %d, received %d)\n",
@@ -490,22 +467,19 @@ free_bad_msg:
 			/* free message */
 			msgb_free(msg_tch);
 			msg_tch = NULL;
-			goto send_frame;
 		}
 	}
 
-send_frame:
-	*_msg_tch = msg_tch;
-	*_msg_facch = msg_facch;
+	return msg_tch;
 }
 
 /* obtain a to-be-transmitted TCH/F (Full Traffic Channel) burst */
 int tx_tchf_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 {
-	struct msgb *msg_tch = NULL, *msg_facch = NULL;
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[br->chan];
 	uint8_t tch_mode = chan_state->tch_mode;
 	ubit_t *burst, **bursts_p = &chan_state->dl_bursts;
+	struct msgb *msg;
 
 	/* send burst, if we already got a frame */
 	if (br->bid > 0) {
@@ -514,14 +488,12 @@ int tx_tchf_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 		goto send_burst;
 	}
 
-	tx_tch_common(l1ts, br, &msg_tch, &msg_facch);
-
 	/* BURST BYPASS */
 
 	/* allocate burst memory, if not already,
 	 * otherwise shift buffer by 4 bursts for interleaving */
 	if (!*bursts_p) {
-		*bursts_p = talloc_zero_size(tall_bts_ctx, 928);
+		*bursts_p = talloc_zero_size(l1ts, 928);
 		if (!*bursts_p)
 			return -ENOMEM;
 	} else {
@@ -529,8 +501,11 @@ int tx_tchf_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 		memset(*bursts_p + 464, 0, 464);
 	}
 
+	/* dequeue a message to be transmitted */
+	msg = tch_dl_dequeue(l1ts, br);
+
 	/* no message at all, send a dummy L2 frame on FACCH */
-	if (!msg_tch && !msg_facch) {
+	if (msg == NULL) {
 		static const uint8_t dummy[GSM_MACBLOCK_LEN] = {
 			0x03, 0x03, 0x01, /* TODO: use randomized padding */
 			0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
@@ -539,31 +514,30 @@ int tx_tchf_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 
 		LOGL1SB(DL1P, LOGL_DEBUG, l1ts, br, "No TCH or FACCH prim for transmit.\n");
 		gsm0503_tch_fr_encode(*bursts_p, dummy, sizeof(dummy), 1);
+		chan_state->dl_facch_bursts = 8;
 		goto send_burst;
 	}
 
-	/* encode bursts (prioritize FACCH) */
-	if (msg_facch) {
-		gsm0503_tch_fr_encode(*bursts_p, msg_facch->l2h, msgb_l2len(msg_facch),
-			1);
+	/* populate the buffer with bursts */
+	if (msgb_l2len(msg) == GSM_MACBLOCK_LEN) {
+		gsm0503_tch_fr_encode(*bursts_p, msg->l2h, msgb_l2len(msg), 1);
 		chan_state->dl_facch_bursts = 8;
-	} else if (tch_mode == GSM48_CMODE_SPEECH_AMR)
+	} else if (tch_mode == GSM48_CMODE_SPEECH_AMR) {
 		/* the first FN 4,13,21 defines that CMI is included in frame,
 		 * the first FN 0,8,17 defines that CMR is included in frame.
 		 */
-		gsm0503_tch_afs_encode(*bursts_p, msg_tch->l2h + 2,
-			msgb_l2len(msg_tch) - 2, !dl_amr_fn_is_cmi(br->fn),
+		gsm0503_tch_afs_encode(*bursts_p, msg->l2h + sizeof(struct amr_hdr),
+			msgb_l2len(msg) - sizeof(struct amr_hdr),
+			!sched_tchf_dl_amr_cmi_map[br->fn % 26],
 			chan_state->codec, chan_state->codecs,
 			chan_state->dl_ft,
 			chan_state->dl_cmr);
-	else
-		gsm0503_tch_fr_encode(*bursts_p, msg_tch->l2h, msgb_l2len(msg_tch), 1);
+	} else {
+		gsm0503_tch_fr_encode(*bursts_p, msg->l2h, msgb_l2len(msg), 1);
+	}
 
 	/* free message */
-	if (msg_tch)
-		msgb_free(msg_tch);
-	if (msg_facch)
-		msgb_free(msg_facch);
+	msgb_free(msg);
 
 send_burst:
 	/* compose burst */
