@@ -35,6 +35,8 @@
 #include <osmocom/coding/gsm0503_coding.h>
 #include <osmocom/coding/gsm0503_amr_dtx.h>
 
+#include <osmocom/netif/amr.h>
+
 #include <osmo-bts/bts.h>
 #include <osmo-bts/l1sap.h>
 #include <osmo-bts/logging.h>
@@ -64,6 +66,18 @@ static const uint8_t sched_tchh_ul_amr_cmi_map[26] = {
 	[3]  = 1, /* TCH/H(1): a=18 / d=24 / f=3 */
 };
 
+/* TDMA frame number of burst 'a' should be used as the table index.
+ * This mapping is valid for both FACCH/H(0) and FACCH/H(1). */
+const uint8_t sched_tchh_dl_amr_cmi_map[26] = {
+	[4]  = 1, /* TCH/H(0): a=4 */
+	[13] = 1, /* TCH/H(0): a=13 */
+	[21] = 1, /* TCH/H(0): a=21 */
+
+	[5]  = 1, /* TCH/H(1): a=5 */
+	[14] = 1, /* TCH/H(1): a=14 */
+	[22] = 1, /* TCH/H(1): a=22 */
+};
+
 /* 3GPP TS 45.002, table 1 in clause 7: Mapping tables.
  * TDMA frame number of burst 'f' is always used as the table index. */
 static const uint8_t sched_tchh_ul_facch_map[26] = {
@@ -196,7 +210,7 @@ int rx_tchh_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		fn_is_cmi = sched_tchh_ul_amr_cmi_map[bi->fn % 26];
 
 		/* See comment in function rx_tchf_fn() */
-		amr = 2;
+		amr = sizeof(struct amr_hdr);
 		rc = gsm0503_tch_ahs_decode_dtx(tch_data + amr, *bursts_p,
 						!sched_tchh_ul_facch_map[bi->fn % 26],
 						!fn_is_cmi, chan_state->codec,
@@ -336,7 +350,7 @@ bfi:
 					       "not sending BFI\n", rc);
 					return -EINVAL;
 				}
-				memset(tch_data + 2, 0, rc - 2);
+				memset(tch_data + sizeof(struct amr_hdr), 0, rc - sizeof(struct amr_hdr));
 				break;
 			default:
 				LOGL1SB(DL1P, LOGL_ERROR, l1ts, bi,
@@ -432,17 +446,19 @@ int tx_tchh_fn(struct l1sched_ts *l1ts, struct trx_dl_burst_req *br)
 		gsm0503_tch_hr_encode(*bursts_p, msg->l2h, msgb_l2len(msg));
 		chan_state->dl_ongoing_facch = 1; /* first of two TCH frames */
 		chan_state->dl_facch_bursts = 6;
-	} else if (tch_mode == GSM48_CMODE_SPEECH_AMR)
+	} else if (tch_mode == GSM48_CMODE_SPEECH_AMR) {
 		/* the first FN 4,13,21 or 5,14,22 defines that CMI is included
 		 * in frame, the first FN 0,8,17 or 1,9,18 defines that CMR is
 		 * included in frame. */
-		gsm0503_tch_ahs_encode(*bursts_p, msg->l2h + 2,
-			msgb_l2len(msg) - 2, !dl_amr_fn_is_cmi(br->fn),
+		gsm0503_tch_ahs_encode(*bursts_p, msg->l2h + sizeof(struct amr_hdr),
+			msgb_l2len(msg) - sizeof(struct amr_hdr),
+			!sched_tchh_dl_amr_cmi_map[br->fn % 26],
 			chan_state->codec, chan_state->codecs,
 			chan_state->dl_ft,
 			chan_state->dl_cmr);
-	else
+	} else {
 		gsm0503_tch_hr_encode(*bursts_p, msg->l2h, msgb_l2len(msg));
+	}
 
 	/* free message */
 	msgb_free(msg);
