@@ -71,7 +71,7 @@ int rx_data_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 
 	/* allocate burst memory, if not already */
 	if (!*bursts_p) {
-		*bursts_p = talloc_zero_size(tall_bts_ctx, 464);
+		*bursts_p = talloc_zero_size(l1ts, 464);
 		if (!*bursts_p)
 			return -ENOMEM;
 	}
@@ -79,7 +79,7 @@ int rx_data_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 	/* UL-SACCH requires additional memory to keep a copy of each previous
 	 * burst set. */
 	if (L1SAP_IS_LINK_SACCH(trx_chan_desc[bi->chan].link_id) && !chan_state->ul_bursts_prev) {
-		chan_state->ul_bursts_prev = talloc_zero_size(tall_bts_ctx, 464);
+		chan_state->ul_bursts_prev = talloc_zero_size(l1ts, 464);
 		if (!chan_state->ul_bursts_prev)
 			return -ENOMEM;
 	}
@@ -111,7 +111,7 @@ int rx_data_fn(struct l1sched_ts *l1ts, const struct trx_ul_burst_ind *bi)
 		return 0;
 
 	/* average measurements of the last 4 bursts */
-	trx_sched_meas_avg(chan_state, &meas_avg, SCHED_MEAS_AVG_M_QUAD);
+	trx_sched_meas_avg(chan_state, &meas_avg, SCHED_MEAS_AVG_M_S4N4);
 
 	/* check for complete set of bursts */
 	if ((*mask & 0xf) != 0xf) {
@@ -207,24 +207,9 @@ got_msg:
 
 	/* BURST BYPASS */
 
-	/* handle loss detection of SACCH */
-	if (L1SAP_IS_LINK_SACCH(trx_chan_desc[br->chan].link_id)) {
-		/* count and send BFI */
-		if (++(l1ts->chan_state[br->chan].lost_frames) > 1) {
-			/* TODO: Should we pass old TOA here? Otherwise we risk
-			 * unnecessary decreasing TA */
-
-			/* Note: RSSI is set to 0 to indicate to the higher
-			 * layers that this is a faked ph_data_ind */
-			_sched_compose_ph_data_ind(l1ts, 0, br->chan, NULL, 0,
-						   0, 0, 0, 10000,
-						   PRES_INFO_INVALID);
-		}
-	}
-
 	/* allocate burst memory, if not already */
 	if (!*bursts_p) {
-		*bursts_p = talloc_zero_size(tall_bts_ctx, 464);
+		*bursts_p = talloc_zero_size(l1ts, 464);
 		if (!*bursts_p)
 			return -ENOMEM;
 	}
