@@ -22,7 +22,6 @@
 #pragma once
 
 #include <stdint.h>
-#include <errno.h>
 
 extern void *tall_bts_ctx;
 
@@ -34,9 +33,3 @@ static inline uint16_t compute_ber10k(int n_bits_total, int n_errors)
 	else
 		return 10000 * n_errors / n_bits_total;
 }
-
-/* determine if the FN is transmitting a CMR (1) or not (0) */
-static inline int fn_is_codec_mode_request(uint32_t fn)
-{
-	return (((fn + 4) % 26) >> 2) & 1;
-}
