@@ -118,8 +118,7 @@ static void lchan_report_interf_meas(const struct gsm_lchan *lchan)
 	gsm_lchan_interf_meas_push((struct gsm_lchan *) lchan, interf_avg);
 }
 
-static void bts_report_interf_meas(const struct gsm_bts *bts,
-				   const uint32_t fn)
+static void bts_report_interf_meas(const struct gsm_bts *bts)
 {
 	const struct gsm_bts_trx *trx;
 	unsigned int tn, ln;
@@ -148,9 +147,12 @@ static struct phy_instance *dlfh_route_br(const struct trx_dl_burst_req *br,
 	if (ts->fh_trx_list[idx] != NULL)
 		return ts->fh_trx_list[idx]->pinst;
 
+	struct bts_trx_priv *priv = (struct bts_trx_priv *) ts->trx->bts->model_priv;
+
 	/* The "cache" may not be filled yet, lookup the transceiver */
 	llist_for_each_entry(trx, &ts->trx->bts->trx_list, list) {
 		if (trx->arfcn == ts->hopping.arfcn_list[idx]) {
+			rate_ctr_inc2(priv->ctrs, BTSTRX_CTR_SCHED_DL_FH_CACHE_MISS);
 			ts->fh_trx_list[idx] = trx;
 			return trx->pinst;
 		}
@@ -160,8 +162,7 @@ static struct phy_instance *dlfh_route_br(const struct trx_dl_burst_req *br,
 		"for a Downlink burst (fn=%u, tn=%u, " SCHED_FH_PARAMS_FMT ")\n",
 		br->fn, br->tn, SCHED_FH_PARAMS_VALS(ts));
 
-	struct bts_trx_priv *priv = (struct bts_trx_priv *) ts->trx->bts->model_priv;
-	rate_ctr_inc(rate_ctr_group_get_ctr(priv->ctrs, BTSTRX_CTR_SCHED_DL_FH_NO_CARRIER));
+	rate_ctr_inc2(priv->ctrs, BTSTRX_CTR_SCHED_DL_FH_NO_CARRIER);
 
 	return NULL;
 }
@@ -268,7 +269,7 @@ static void bts_sched_fn(struct gsm_bts *bts, const uint32_t fn)
 
 	/* Report interference measurements */
 	if (fn % 104 == 0) /* SACCH period */
-		bts_report_interf_meas(bts, fn);
+		bts_report_interf_meas(bts);
 
 	/* send time indication */
 	l1if_mph_time_ind(bts, fn);
@@ -347,7 +348,7 @@ static struct gsm_bts_trx *ulfh_route_bi(const struct trx_ul_burst_ind *bi,
 		bi->fn, bi->tn, SCHED_FH_PARAMS_VALS(&src_trx->ts[bi->tn]));
 
 	struct bts_trx_priv *priv = (struct bts_trx_priv *) src_trx->bts->model_priv;
-	rate_ctr_inc(rate_ctr_group_get_ctr(priv->ctrs, BTSTRX_CTR_SCHED_UL_FH_NO_CARRIER));
+	rate_ctr_inc2(priv->ctrs, BTSTRX_CTR_SCHED_UL_FH_NO_CARRIER);
 
 	return NULL;
 }
@@ -621,6 +622,7 @@ void trx_sched_meas_push(struct l1sched_chan_state *chan_state,
 	unsigned int current = chan_state->meas.current;
 
 	chan_state->meas.buf[current] = (struct l1sched_meas_set) {
+		.fn = bi->fn,
 		.ci_cb = (bi->flags & TRX_BI_F_CI_CB) ? bi->ci_cb : 0,
 		.toa256 = bi->toa256,
 		.rssi = bi->rssi,
@@ -629,6 +631,17 @@ void trx_sched_meas_push(struct l1sched_chan_state *chan_state,
 	chan_state->meas.current = (current + 1) % hist_size;
 }
 
+/* Measurement averaging mode sets: [MODE] = { SHIFT, NUM } */
+static const uint8_t trx_sched_meas_modeset[][2] = {
+	[SCHED_MEAS_AVG_M_S4N4] = { 4, 4 },
+	[SCHED_MEAS_AVG_M_S8N8] = { 8, 8 },
+	[SCHED_MEAS_AVG_M_S6N4] = { 6, 4 },
+	[SCHED_MEAS_AVG_M_S6N6] = { 6, 6 },
+	[SCHED_MEAS_AVG_M_S8N4] = { 8, 4 },
+	[SCHED_MEAS_AVG_M_S6N2] = { 6, 2 },
+	[SCHED_MEAS_AVG_M_S4N2] = { 4, 2 },
+};
+
 /* Calculate the AVG of n measurements from the history */
 void trx_sched_meas_avg(const struct l1sched_chan_state *chan_state,
 			struct l1sched_meas_set *avg,
@@ -637,44 +650,17 @@ void trx_sched_meas_avg(const struct l1sched_chan_state *chan_state,
 	unsigned int hist_size = ARRAY_SIZE(chan_state->meas.buf);
 	unsigned int current = chan_state->meas.current;
 	const struct l1sched_meas_set *set;
-	unsigned int shift, pos, i, n;
+	unsigned int pos, i;
 
 	float rssi_sum = 0;
 	int toa256_sum = 0;
 	int ci_cb_sum = 0;
 
-	switch (mode) {
-	/* last 4 bursts (default for xCCH, TCH/H, PTCCH and PDTCH) */
-	case SCHED_MEAS_AVG_M_QUAD:
-		n = 4; shift = n;
-		break;
-	/* last 8 bursts (default for TCH/F and FACCH/F) */
-	case SCHED_MEAS_AVG_M_OCTO:
-		n = 8; shift = n;
-		break;
-	/* last 6 bursts (default for FACCH/H) */
-	case SCHED_MEAS_AVG_M_SIX:
-		n = 6; shift = n;
-		break;
-	/* first 4 of last 8 bursts */
-	case SCHED_MEAS_AVG_M8_FIRST_QUAD:
-		n = 4; shift = 8;
-		break;
-	/* first 2 of last 6 bursts */
-	case SCHED_MEAS_AVG_M6_FIRST_TWO:
-		n = 2; shift = 6;
-		break;
-	/* middle 2 of last 6 bursts */
-	case SCHED_MEAS_AVG_M6_MIDDLE_TWO:
-		n = 2; shift = 4;
-		break;
-	default:
-		/* Shall not happen */
-		OSMO_ASSERT(false);
-	}
+	const unsigned int shift = trx_sched_meas_modeset[mode][0];
+	const unsigned int num = trx_sched_meas_modeset[mode][1];
 
 	/* Calculate the sum of n entries starting from pos */
-	for (i = 0; i < n; i++) {
+	for (i = 0; i < num; i++) {
 		pos = (current + hist_size - shift + i) % hist_size;
 		set = &chan_state->meas.buf[pos];
 
@@ -683,14 +669,34 @@ void trx_sched_meas_avg(const struct l1sched_chan_state *chan_state,
 		ci_cb_sum  += set->ci_cb;
 	}
 
+	/* First sample contains TDMA frame number of the first burst */
+	pos = (current + hist_size - shift) % hist_size;
+	set = &chan_state->meas.buf[pos];
+
 	/* Calculate the average for each value */
 	*avg = (struct l1sched_meas_set) {
-		.rssi   = (rssi_sum   / n),
-		.toa256 = (toa256_sum / n),
-		.ci_cb  = (ci_cb_sum  / n),
+		.fn     = set->fn, /* first burst */
+		.rssi   = (rssi_sum   / num),
+		.toa256 = (toa256_sum / num),
+		.ci_cb  = (ci_cb_sum  / num),
 	};
 
-	LOGP(DMEAS, LOGL_DEBUG, "Measurement AVG (num=%u, shift=%u): "
-	     "RSSI %f, ToA256 %d, C/I %d cB\n", n, shift,
-	     avg->rssi, avg->toa256, avg->ci_cb);
+	LOGP(DMEAS, LOGL_DEBUG, "%s%sMeasurement AVG (num=%u, shift=%u): "
+	     "RSSI %f, ToA256 %d, C/I %d cB\n",
+	     chan_state->lchan ? gsm_lchan_name(chan_state->lchan) : "",
+	     chan_state->lchan ? " " : "",
+	     num, shift, avg->rssi, avg->toa256, avg->ci_cb);
+}
+
+/* Lookup TDMA frame number of the N-th sample in the history */
+uint32_t trx_sched_lookup_fn(const struct l1sched_chan_state *chan_state,
+			     const unsigned int shift)
+{
+	const unsigned int hist_size = ARRAY_SIZE(chan_state->meas.buf);
+	const unsigned int current = chan_state->meas.current;
+	unsigned int pos;
+
+	/* First sample contains TDMA frame number of the first burst */
+	pos = (current + hist_size - shift) % hist_size;
+	return chan_state->meas.buf[pos].fn;
 }
