@@ -685,8 +685,7 @@ static int trx_ctrl_read_cb(struct osmo_fd *ofd, unsigned int what)
 	LOGPPHI(l1h->phy_inst, DTRX, LOGL_INFO, "Response message: '%s'\n", buf);
 
 	/* abort timer and send next message, if any */
-	if (osmo_timer_pending(&l1h->trx_ctrl_timer))
-		osmo_timer_del(&l1h->trx_ctrl_timer);
+	osmo_timer_del(&l1h->trx_ctrl_timer);
 
 	/* get command for response message */
 	if (llist_empty(&l1h->trx_ctrl_list)) {
@@ -827,13 +826,18 @@ static inline int trx_data_parse_mts(struct phy_instance *phy_inst,
 
 	/* | 7 6 5 4 3 2 1 0 | Bitmask / description
 	 * | . 0 0 X X . . . | GMSK, 4 TSC sets (0..3)
-	 * | . 0 1 0 X . . . | 8-PSK, 2 TSC sets (0..1) */
+	 * | . 0 1 0 X . . . | 8-PSK, 2 TSC sets (0..1)
+	 * | . 0 1 1 0 . . . | GMSK, Access Burst */
 	if ((mts >> 5) == 0x00) {
 		bi->mod = TRX_MOD_T_GMSK;
 		bi->tsc_set = (mts >> 3) & 0x03;
 	} else if ((mts >> 4) == 0x02) {
 		bi->mod = TRX_MOD_T_8PSK;
 		bi->tsc_set = (mts >> 3) & 0x01;
+	} else if ((mts >> 3) == 0x06) {
+		bi->flags |= TRX_BI_F_ACCESS_BURST;
+		bi->mod = TRX_MOD_T_GMSK;
+		bi->tsc_set = 0;
 	} else {
 		LOGPPHI(phy_inst, DTRX, LOGL_ERROR,
 			"Rx TRXD PDU with unknown or not supported "
@@ -862,7 +866,7 @@ static int trx_data_handle_hdr_v1(struct phy_instance *phy_inst,
 
 	/* MTS (Modulation and Training Sequence) */
 	rc = trx_data_parse_mts(phy_inst, bi, buf[0]);
-	if (rc < 0)
+	if (OSMO_UNLIKELY(rc < 0))
 		return rc;
 
 	/* C/I: Carrier-to-Interference ratio (in centiBels) */
@@ -893,7 +897,7 @@ static int trx_data_handle_pdu_v2(struct phy_instance *phy_inst,
 
 	/* MTS (Modulation and Training Sequence) */
 	rc = trx_data_parse_mts(phy_inst, bi, buf[2]);
-	if (rc < 0)
+	if (OSMO_UNLIKELY(rc < 0))
 		return rc;
 
 	bi->rssi = -(int8_t)buf[3];
@@ -903,7 +907,7 @@ static int trx_data_handle_pdu_v2(struct phy_instance *phy_inst,
 
 	/* TDMA frame number is absent in batched PDUs */
 	if (bi->_num_pdus == 0) {
-		if (buf_len < sizeof(bi->fn) + TRX_UL_V2HDR_LEN) {
+		if (OSMO_UNLIKELY(buf_len < sizeof(bi->fn) + TRX_UL_V2HDR_LEN)) {
 			LOGPPHI(phy_inst, DTRX, LOGL_ERROR,
 				"Rx malformed TRXDv2 PDU: not enough bytes "
 				"to parse TDMA frame number\n");
@@ -936,7 +940,7 @@ static int trx_data_handle_burst(struct trx_ul_burst_ind *bi,
 	};
 
 	bi->burst_len = bl[bi->mod];
-	if (buf_len < bi->burst_len)
+	if (OSMO_UNLIKELY(buf_len < bi->burst_len))
 		return -EINVAL;
 
 	/* Convert unsigned soft-bits [254..0] to soft-bits [-127..127] */
@@ -1009,7 +1013,7 @@ static int trx_data_read_cb(struct osmo_fd *ofd, unsigned int what)
 	uint8_t pdu_ver;
 
 	buf_len = recv(ofd->fd, trx_data_buf, sizeof(trx_data_buf), 0);
-	if (buf_len <= 0) {
+	if (OSMO_UNLIKELY(buf_len <= 0)) {
 		strerror_r(errno, (char *) trx_data_buf, sizeof(trx_data_buf));
 		LOGPPHI(l1h->phy_inst, DTRX, LOGL_ERROR,
 			"recv() failed on TRXD with rc=%zd (%s)\n",
@@ -1021,7 +1025,7 @@ static int trx_data_read_cb(struct osmo_fd *ofd, unsigned int what)
 	pdu_ver = buf[0] >> 4;
 
 	/* Make sure that PDU version matches our expectations */
-	if (pdu_ver != l1h->config.trxd_pdu_ver_use) {
+	if (OSMO_UNLIKELY(pdu_ver != l1h->config.trxd_pdu_ver_use)) {
 		LOGPPHI(l1h->phy_inst, DTRX, LOGL_ERROR,
 			"Rx TRXD PDU with unexpected version %u (expected %u)\n",
 			pdu_ver, l1h->config.trxd_pdu_ver_use);
@@ -1037,7 +1041,7 @@ static int trx_data_read_cb(struct osmo_fd *ofd, unsigned int what)
 		bi.flags = 0x00;
 
 		/* Make sure that we have enough bytes to parse the header */
-		if (buf_len < trx_data_rx_hdr_len[pdu_ver]) {
+		if (OSMO_UNLIKELY(buf_len < trx_data_rx_hdr_len[pdu_ver])) {
 			LOGPPHI(l1h->phy_inst, DTRX, LOGL_ERROR,
 				"Rx malformed TRXDv%u PDU: len=%zd < expected %u\n",
 				pdu_ver, buf_len, trx_data_rx_hdr_len[pdu_ver]);
@@ -1061,10 +1065,10 @@ static int trx_data_read_cb(struct osmo_fd *ofd, unsigned int what)
 		}
 
 		/* Header parsing error */
-		if (hdr_len < 0)
+		if (OSMO_UNLIKELY(hdr_len < 0))
 			return hdr_len;
 
-		if (bi.fn >= GSM_TDMA_HYPERFRAME) {
+		if (OSMO_UNLIKELY(bi.fn >= GSM_TDMA_HYPERFRAME)) {
 			LOGPPHI(l1h->phy_inst, DTRX, LOGL_ERROR,
 				"Rx malformed TRXDv%u PDU: illegal TDMA fn=%u\n",
 				pdu_ver, bi.fn);
@@ -1076,7 +1080,7 @@ static int trx_data_read_cb(struct osmo_fd *ofd, unsigned int what)
 		buf += hdr_len;
 
 		/* Calculate burst length and parse it (if present) */
-		if (trx_data_handle_burst(&bi, buf, buf_len) != 0) {
+		if (OSMO_UNLIKELY(trx_data_handle_burst(&bi, buf, buf_len) != 0)) {
 			LOGPPHI(l1h->phy_inst, DTRX, LOGL_ERROR,
 				"Rx malformed TRXDv%u PDU: odd burst length=%zd\n",
 				pdu_ver, buf_len);
@@ -1117,7 +1121,7 @@ int trx_if_send_burst(struct trx_l1h *l1h, const struct trx_dl_burst_req *br)
 	ssize_t snd_len, buf_len;
 
 	/* Make sure that the PHY is powered on */
-	if (!trx_if_powered(l1h)) {
+	if (OSMO_UNLIKELY(!trx_if_powered(l1h))) {
 		LOGPPHI(l1h->phy_inst, DTRX, LOGL_ERROR,
 			"Ignoring Tx data, transceiver is powered off\n");
 		return -ENODEV;
@@ -1190,7 +1194,7 @@ sendall:
 	pdu_num = 0;
 
 	snd_len = send(l1h->trx_ofd_data.fd, trx_data_buf, buf_len, 0);
-	if (snd_len <= 0) {
+	if (OSMO_UNLIKELY(snd_len <= 0)) {
 		strerror_r(errno, (char *) trx_data_buf, sizeof(trx_data_buf));
 		LOGPPHI(l1h->phy_inst, DTRX, LOGL_ERROR,
 			"send() failed on TRXD with rc=%zd (%s)\n",
@@ -1222,8 +1226,7 @@ void trx_if_flush(struct trx_l1h *l1h)
 	l1h->last_acked = NULL;
 
 	/* Tx queue is now empty, so there's no point in keeping the retrans timer armed: */
-	if (osmo_timer_pending(&l1h->trx_ctrl_timer))
-		osmo_timer_del(&l1h->trx_ctrl_timer);
+	osmo_timer_del(&l1h->trx_ctrl_timer);
 }
 
 /*! close the TRX for given handle (data + control socket) */
